#!/bin/bash
TYPE=$1
PACKAGE_RELEASE=$2
PACKAGE_BUILD=$3

# The base directory where to look for packages to install
LOCAL_PACKAGE_BASE="/vagrant/packages"

function detect_distro
{
  # If the host has /etc/release we should use that.
  if [ -e /etc/os-release ]; then
    echo "Using /etc/os-release for distro information"
    source /etc/os-release
  else
    echo "Could not find /etc/os-release, moving on"
    if [ -e /etc/centos-release ]; then
      echo "Found /etc/centos-release"
      ID="centos"
    elif [ -e /etc/redhat-release ]; then
      echo "Found /etc/redhat-release"
      ID="redhat"
    else
      echo "Unable to detect distro"
    fi
  fi
  case $ID in
       "redhat"|"centos")
         detect_redhat_and_centos
         ;;
        *)
          echo "Unable to detect distro, will try quick install"
         ;;
  esac


  #echo ID=$ID
  #echo VERSION_ID=$VERSION_ID
}

function detect_redhat_and_centos
{
  if [ -e /etc/os-release ]; then
    VERSION_ID_MAJOR=$(echo $VERSION_ID|cut -d"." -f "1")
  elif [ -e /etc/centos-release ]; then
    NAME="CentOS Linux"
    ID="centos"
    VERSION_ID=$(grep -P "(\d.*)\s" --only-matching /etc/centos-release)
    VERSION_ID_MAJOR=$(echo $VERSION_ID|cut -d"." -f "1")
  elif [ -e /etc/redhat-release ]; then
    NAME="RedHat Linux"
    ID="centos"
    VERSION_ID=$(grep -P "(\d.*)\s" --only-matching /etc/redhat-release)
    VERSION_ID_MAJOR=$(echo $VERSION_ID|cut -d"." -f "1")
  else
    echo "Unable to detect redhat or centos details"
  fi

  echo NAME=$NAME
  echo ID=$ID
  echo VERSION_ID=$VERSION_ID
  echo VERSION_ID_MAJOR=$VERSION_ID_MAJOR
}

function set_rpm_package_details
{
  ARCH=$(uname -i)

  if [ "$TYPE" == "agent" ]; then
    PKG_NAME=cfengine-nova
  elif [ "$TYPE" == "hub" ]; then
    PKG_NAME=cfengine-nova-hub
    # This is not necessary after 3.9.1
    # rpm -q libtool-ltdl || yum -y install libtool-ltdl
  fi

  case $ID in
    "redhat"|"centos")
      if [ "$VERSION_ID_MAJOR" -ge 7 ]; then
        # Use the correct package for the type
        if [ "$TYPE" == "agent" ]; then
		  PKG_DIR=agent_rhel7_x86_64
          PKG=$PKG_NAME-$PACKAGE_RELEASE-$PACKAGE_BUILD.el7.$ARCH.rpm
	    else
	      PKG_DIR=redhat_7_x86_64
          PKG=$PKG_NAME-$PACKAGE_RELEASE-$PACKAGE_BUILD.$ARCH.rpm
        fi

      elif [ "$VERSION_ID_MAJOR" -lt 7 ]; then
        if [ "$TYPE" == "agent" ]; then
          PKG_DIR=agent_rpm_x86_64
          PKG=$PKG_NAME-$PACKAGE_RELEASE-$PACKAGE_BUILD.el5.$ARCH.rpm
        elif [ "$TYPE" == "hub" ]; then
          if [ "$VERSION_ID_MAJOR" -eq 6 ]; then
		    PKG_DIR=redhat_6_x86_64
          else
            echo "Error: Unable to select PKG_DIR for $ID $VERSION_ID_MAJOR $TYPE"
            exit 1
          fi
          PKG=$PKG_NAME-$PACKAGE_RELEASE-$PACKAGE_BUILD.$ARCH.rpm
	    else
	      echo "ERROR: Unable to select PKG_DIR"
		  exit 1
            fi

         fi
	 ;;
        "sles")
	  echo "SLES SUPPORT NOT IMPLEMENTED"
	  exit 1
         ;;
  esac

  echo PKG_DIR="$PKG_DIR"
  echo PKG="$PKG"
}

function check_pkg_presence
{
  _LOCAL_PACKAGE_BASE=$1
  _PKG_DIR=$2
  _PKG=$3
  _PACKAGE_RELEASE=$4
  _TYPE=$5

  if [ ! -e $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG ]; then
    echo "Could not find $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG"
    mkdir -p $_LOCAL_PACKAGE_BASE/$_PKG_DIR
    echo "Fetching $_PKG"
    if which wget; then
      echo using wget
      echo wget --quiet http://cfengine-package-repos.s3.amazonaws.com/enterprise/Enterprise-$_PACKAGE_RELEASE/$_TYPE/$_PKG_DIR/$_PKG -O $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG
      wget --quiet http://cfengine-package-repos.s3.amazonaws.com/enterprise/Enterprise-$_PACKAGE_RELEASE/$_TYPE/$_PKG_DIR/$_PKG -O $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG
    else
      echo wget not found, falling back to curl
      echo curl -s http://cfengine-package-repos.s3.amazonaws.com/enterprise/Enterprise-$_PACKAGE_RELEASE/$_TYPE/$_PKG_DIR/$_PKG -o $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG
      curl -s http://cfengine-package-repos.s3.amazonaws.com/enterprise/Enterprise-$_PACKAGE_RELEASE/$_TYPE/$_PKG_DIR/$_PKG -o $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG
    fi
  else
    echo "Using $_LOCAL_PACKAGE_BASE/$_PKG_DIR/$_PKG"
  fi
}

function install_rpm_pkg
{
  if ! /bin/rpm -q $PKG_NAME ; then
    echo "Installing CFEngine Enterprise $TYPE"
    /bin/rpm -U $LOCAL_PACKAGE_BASE/$PKG_DIR/$PKG
  else
    echo "$PKG_NAME already installed, skipping installation"
  fi
}

function select_package
{
  case $ID in
       "redhat"|"centos")
         set_rpm_package_details
	 check_pkg_presence $LOCAL_PACKAGE_BASE $PKG_DIR $PKG $PACKAGE_RELEASE $TYPE
	 install_rpm_pkg
         ;;
       *)
         echo "Only redhat/centos is built-in to vagrant quickstart, attempting quick install script"
         if which wget > /dev/null; then
             wget https://s3.amazonaws.com/cfengine.packages/quick-install-cfengine-enterprise.sh
         elif which curl > /dev/null; then
             curl https://s3.amazonaws.com/cfengine.packages/quick-install-cfengine-enterprise.sh -O
         else
             echo "Apparently no wget or curl present, cannot download quickinstall script"
             exit 1
         fi
         CFEngine_Enterprise_Package_Version=$PACKAGE_RELEASE bash quick-install-cfengine-enterprise.sh $TYPE
	 ;;
  esac
}

detect_distro
select_package
