let waitForSpinner = function () {
    let def = $.Deferred();
    let interval = setInterval(function () {
        if ($('#globalSpinner').css('display') == 'none') {
            clearInterval(interval);
            def.resolve();
        }
    }, 1000);
    return def;
};

let hostUrl;
if (window.localStorage.getItem('tourStarted') == 'true' && window.localStorage.getItem('tour_end') != 'yes') {
    $.ajax({
        url: "/astrolabe/host",
        async: false,
        type: 'POST',
        data: {page: 1, count: 1},
        dataType: 'json',
        success: function (data) {
            // if no hosts found then open page with undefined hostkey
            // which still shows host info page without specific host data
            hostUrl = `/assets/host_assets/${data[0].id ?? 'undefined'}`;
        }
    });
}


var tourGuide = new Tour({
    backdrop: true,
    backdropPadding: 5,
    onEnd: function () {
        window.location.href = '/dashboard';
    },
    template: '<div class="popover" role="tooltip"> ' +
        '<div class="arrow"></div> ' +
        '<h3 class="popover-title"></h3> ' +
        '<div class="popover-content"></div> ' +
        '<div class="popover-navigation"> <div class="btn-group  pull-left"> ' +
        '<button class="btn btn-sm btn-default" data-role="prev"><i class="icon-arrow-left"></i> Prev</button> ' +
        '<button class="btn btn-sm btn-default" data-role="next">Next <i class="icon-arrow-right"></i></button></div>' +
        '<div class="btn-group  pull-right"> ' +
        '<button class="btn btn-sm pause-button" data-role="pause-tour"><i class="icon-pause"></i> Pause</button> ' +
        '<button class="btn btn-sm  end-button" data-role="end"><i class="icon-stop"></i> End tour</button> </div>  </div> '
});

const openDropdownMenu = function () {
    let def = $.Deferred();

    let interval = setInterval(function () {
        if (
            $('#globalSpinner').css('display') == 'none' &&
            $('.pull-right.dropdown.open').length > 0
        ) {
            clearInterval(interval);
            def.resolve();
        } else {
            $('#dropdownMenu').click();
        }
    }, 1000);
    return def.promise();
}

const waitElementLoading = function (element) {
    let def = $.Deferred();
    let interval = setInterval(function () {
        if ($(element).is(":visible") == true) {
            clearInterval(interval);
            def.resolve();
        }
    }, 1000);
    return def.promise();
}

let steps = [
        {
            element: ".selectedElement",
            title: "Added vcs type dropdown to the Version control repository settings",
            content: "You can select between: <code>Git</code> or <code>Git repo with cfbs project</code>",
            placement: "bottom",
            path: "/settings/vcs",
            onShown: function () {
                const step = document.getElementById('step-0');
                step.style.width = '435px';
                step.style.maxWidth = '435px';
            }
        },
        {
            element: 'button[ng-click="import()"]',
            title: "Added import/export functionality for compliance reports",
            placement: "bottom",
            path: "/reports/compliance",
            onShow: function () {
                return waitForSpinner().promise();
            },
        },
        {
            element: "div[cmdb]",
            title: "Configuration management database",
            content: "You can now assign data (variables and classes) to individual hosts",
            placement: "right",
            path: hostUrl,
            onShow: function () {
                return waitElementLoading('.cmdb--variables');
            },
        },
        {
            element: "#compliance_menu",
            title: "Compliance reports",
            content: "Set compliance criteria and get a report about the overall compliance level",
            placement: "bottom",
            path: "/reports",
            onShow: function () {
                return waitForSpinner().promise();
            },
        },
        {
            element: ".request-report-collection",
            title: "Trigger report collection",
            content: "Trigger report collection for an individual host from its host info page",
            placement: "left",
            path: hostUrl,
            onShow: function () {
                return waitElementLoading('.cmdb--variables');
            },
        },
        {
            element: ".trigger-agent-run ",
            title: "Trigger agent run",
            content: "Trigger agent run for an individual host from its host info page",
            placement: "left",
            path: hostUrl,
            onShow: function () {
                return waitElementLoading('.cmdb--variables');
            },
        },
        {
            element: ".dark-theme-switch",
            title: "Dark mode",
            content: "Click on the Dark mode switch to enable it",
            placement: "bottom",
            onShow: openDropdownMenu
        },
        {
            element: "#pinned_items_wrapper",
            title: "Pin items",
            content: "You can pin interesting items (inventory, classes, variables) on the host info page. To pin an item " +
                "you need to hover over it and pin icon will appear.",
            placement: "top",
            path: hostUrl,
            onShow: function () {
                return waitElementLoading('.pinned-container');
            },
        },
        {
            element: ".selectedElement",
            title: "Synchronize external directory based groups with Mission Portal roles",
            content: "You can enable LDAP roles syncing with internal Mission Portal roles by clicking " +
                     "the corresponding checkbox on the Authentication settings page.",
            placement: "bottom",
            path: "/settings/authentication",
        },
        {
            element: "li:contains('profile')",
            title: "Time zone setting",
            content: "You can select time zone which will be applied to all reports.",
            placement: "bottom",
            path: "/settings/profile",
            onShow: openDropdownMenu
        },
        {
            element: "ul.dashboard-dropdown",
            title: "Dashboard improvements",
            content: "Clone dashboards shared with you, edit " +
                "and share your copy with other users and or roles",
            placement: "bottom",
            path: "/dashboard#/index",
            onShow: function () {
                let def = $.Deferred();
                let interval = setInterval(function () {
                    if (
                        $('#globalSpinner').css('display') == 'none' &&
                        $('ul.dashboard-dropdown').is(":visible") == true
                    ) {
                        clearInterval(interval);
                        def.resolve();
                    } else {
                        $('a.dashboard-dropdown').click();
                    }
                }, 1000);
                return def.promise()
            },
        },
        {
            element: "div[changes-widget]",
            title: "Resizeable widgets",
            content: "Dashboard widgets can be resized",
            placement: "bottom",
            path: "/dashboard#/index",
            onShow: function () {
                let def = $.Deferred();
                let interval = setInterval(function () {
                    if (
                        $('#globalSpinner').css('display') == 'none' &&
                        $('div[changes-widget]').is(":visible") == true
                    ) {
                        clearInterval(interval);
                        def.resolve();
                    }
                }, 1000);
                return def.promise()
            },
        },
        {
            element: "select[name=\"inventoryChartType\"]",
            title: "New inventory widget types",
            content: "Inventory widgets types: Solid gauge and Table are available for use on dashboards.",
            placement: "bottom",
            path: "/dashboard#/index",
            onShow: function () {
                let def = $.Deferred();
                let interval = setInterval(function () {
                    if (
                        $('#globalSpinner').css('display') == 'none' &&
                        $('select[name="inventoryChartType"]').is(":visible") == true
                    ) {
                        clearInterval(interval);
                        def.resolve();
                    } else {
                        $('.widget.empty a').click();
                        $('div[ng-click="initInventoryForm();"]').click();
                    }
                }, 1000);
                return def.promise()
            }
        }
    ];

steps.map(step => {
    step.onRedirectError = function (tour) {
        tour.next();
    };
    return step;
})

tourGuide.addSteps(steps);

if (window.localStorage.getItem('tourStarted') === 'true' && window.localStorage.getItem('tourPaused') !== 'true') {
    tourGuide.init().start();
}


$(document).ready(function () {
    let resumeButtonHtml = '<button class="btn btn-large resume-tour-guide"><i class="icon-play "></i> Resume tour guide</button>';
    $(document).on('click', '[data-role="pause-tour"]', function () {
        $('.tour-tour').css('visibility', 'hidden');
        $('.tour-backdrop').css('visibility', 'hidden');
        window.localStorage.setItem('tourPaused', 'true');

        $('body').append(resumeButtonHtml)
    });

    if (window.localStorage.getItem('tourPaused') === 'true') {
        $('body').append(resumeButtonHtml)
    }

    $(document).on('click', '.resume-tour-guide', function () {
        window.localStorage.setItem('tourPaused', 'false');
        window.localStorage.removeItem('tour_redirect_to');
        tourGuide.init().start(true);
        $('.tour-tour').css('visibility', 'visible');
        $('.tour-backdrop').css('visibility', 'visible');
        $('.resume-tour-guide').remove();
    });
});
